<?php
/**
 * @package            Joomla
 * @subpackage         Event Booking
 * @author             Tuan Pham Ngoc
 * @copyright          Copyright (C) 2010 - 2023 Ossolution Team
 * @license            GNU/GPL, see LICENSE.php
 */

defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Mail\MailHelper;
use Stripe\Exception\ApiErrorException;
use Stripe\Exception\SignatureVerificationException;
use Stripe\Stripe;
use Stripe\StripeClient;

class os_stripecheckout extends RADPayment
{
	/**
	 * Constructor
	 *
	 * @param   \Joomla\Registry\Registry  $params
	 * @param   array                      $config
	 */
	public function __construct($params, $config = [])
	{
		// Use sandbox API keys if available
		if (!$params->get('mode', 1))
		{
			$keys = [
				'stripe_public_key',
				'stripe_api_key',
				'endpoint_secret',
			];

			foreach ($keys as $key)
			{
				if ($params->get('sandbox_' . $key))
				{
					$params->set($key, $params->get('sandbox_' . $key));
				}
			}
		}

		parent::__construct($params, $config);
	}

	/**
	 * Process payment
	 *
	 * @param   EventbookingTableRegistrant  $row
	 * @param   array                        $data
	 */
	public function processPayment($row, $data)
	{
		$this->loadStripeLib();

		$stripeClient = $this->getStripeClient();
		$app          = Factory::getApplication();

		$Itemid = $app->input->getInt('Itemid', 0);

		$returnUrl = $this->getPaymentCompleteUrl($row, $Itemid, true);
		$cancelUrl = $this->getPaymentCancelUrl($row, $Itemid, true);

		try
		{
			$request = [
				'mode'                => 'payment',
				'line_items'          => [
					[
						'quantity'   => 1,
						'price_data' => [
							'currency'     => $data['currency'],
							'product_data' => [
								'name' => $data['item_name'],
							],
							'unit_amount'  => 100 * round($data['amount'], 2),
						],
					],
				],
				'metadata'            => $this->getMetaData($row, $data),
				'success_url'         => $returnUrl,
				'cancel_url'          => $cancelUrl,
				'client_reference_id' => $this->params->get('order_prefix', 'EB') . $row->id,
				'locale'              => $this->params->get('locale', 'auto'),
				// We might need the lines below to have it works with afterpay_clearpay, see https://stripe.com/docs/api/checkout/sessions/create#create_checkout_session-shipping_address_collection
				/*'shipping_address_collection' => [
					'allowed_countries' => [
						'AU',
						'US',
					],
				],*/
			];

			if (MailHelper::isEmailAddress($row->email))
			{
				$request['customer_email'] = $row->email;
			}

			$session = $stripeClient->checkout->sessions->create($request);

			if (!empty($session->url))
			{
				$app->redirect($session->url);
			}
			else
			{
				$this->redirectToStripe($session->id);
			}
		}
		catch (Exception $e)
		{
			$this->setPaymentErrorMessage($e->getMessage());
			$app->redirect($this->getPaymentFailureUrl($row, $Itemid));
		}
	}

	/**
	 * Verify payment
	 *
	 * @return void
	 * @throws SignatureVerificationException
	 * @throws ApiErrorException
	 */
	public function verifyPayment()
	{
		if (!$this->validate())
		{
			return;
		}

		$id = $this->notificationData['id'];

		$row = $this->getRegistrantTable();

		if (!$row->load($id))
		{
			$this->logGatewayData('Invalid Record ID:' . $id);

			return;
		}

		if ($row->published == 1 && $row->payment_status)
		{
			$this->logGatewayData('Invalid Status, already published before');

			return;
		}

		$this->onPaymentSuccess($row, $this->notificationData['transaction_id']);
	}

	/**
	 * Refund a transaction
	 *
	 * @param   EventbookingTableRegistrant  $row
	 *
	 * @throws Exception
	 */
	public function refund($row)
	{
		$this->loadStripeLib();

		$stripeClient = $this->getStripeClient();

		try
		{
			$stripeClient->refunds->create(['charge' => $row->transaction_id]);
		}
		catch (ApiErrorException $e)
		{
			// Use the variable $error to save any errors
			// To be displayed to the customer later in the page
			$body  = $e->getJsonBody();
			$err   = $body['error'];
			$error = $err['message'];

			throw new Exception($error);
		}
	}

	/**
	 * Validate payment
	 *
	 * @return bool
	 * @throws ApiErrorException
	 * @throws SignatureVerificationException
	 */
	private function validate()
	{
		$this->loadStripeLib();

		Stripe::setApiKey($this->params->get('stripe_api_key'));

		$endpoint_secret = $this->params->get('endpoint_secret');

		$payload    = @file_get_contents('php://input');
		$sig_header = $_SERVER['HTTP_STRIPE_SIGNATURE'];

		try
		{
			$event = \Stripe\Webhook::constructEvent(
				$payload,
				$sig_header,
				$endpoint_secret
			);
		}
		catch (\UnexpectedValueException $e)
		{
			// Invalid payload
			http_response_code(400); // PHP 5.4 or greater

			Factory::getApplication()->close();

			return false;
		}
		catch (SignatureVerificationException $e)
		{
			// Invalid signature
			http_response_code(400); // PHP 5.4 or greater

			Factory::getApplication()->close();

			return false;
		}

		if ($event->type != 'checkout.session.completed')
		{
			return false;
		}

		$stripeClient = $this->getStripeClient();
		$session      = $event->data->object;

		$orderPrefix = $this->params->get('order_prefix', 'EB');

		if ($orderPrefix && strpos($session->client_reference_id, $orderPrefix) === false)
		{
			return false;
		}

		$this->notificationData['id'] = (int) substr($session->client_reference_id, strlen($orderPrefix));

		$paymentIntent = $stripeClient->paymentIntents->retrieve($session->payment_intent);

		$this->notificationData['transaction_id'] = $paymentIntent->latest_charge;

		$this->logGatewayData();

		/* @var EventbookingTableRegistrant $row */
		$row = $this->getRegistrantTable();

		if ($row->load($this->notificationData['id']))
		{
			// Update payment intent description
			$itemName = Text::_('EB_EVENT_REGISTRATION');

			$replaces = EventbookingHelperRegistration::getRegistrationReplaces($row);

			foreach ($replaces as $key => $value)
			{
				$value    = (string) $value;
				$itemName = str_ireplace("[$key]", $value, $itemName);
			}

			$stripeClient->paymentIntents->update($session->payment_intent, ['description' => $itemName]);
		}

		return true;
	}

	/**
	 * Get metadata to pass to Stripe
	 *
	 * @param   EventbookingTableRegistrant  $row
	 * @param   array                        $data
	 *
	 * @return array
	 */
	private function getMetadata($row, $data)
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)
			->select('name, title')
			->from('#__eb_fields')
			->where('published = 1');
		$db->setQuery($query);
		$fields = $db->loadObjectList('name');

		if ($row->first_name && isset($fields['first_name']))
		{
			$metaData[$fields['first_name']->title] = $row->first_name;
		}

		if ($row->last_name && isset($fields['last_name']))
		{
			$metaData[$fields['last_name']->title] = $row->last_name;
		}

		$metaData['Email']  = $row->email;
		$metaData['Source'] = 'Events Booking';

		if ($row->user_id > 0)
		{
			$metaData['User ID'] = $row->user_id;
		}

		$metaData['Registrant ID'] = $row->id;

		return $metaData;
	}

	/**
	 * Load Stripe library, set API Key and make the API ready to be used
	 */
	private function loadStripeLib()
	{
		if (!class_exists('\Stripe\Stripe'))
		{
			require_once __DIR__ . '/stripecheckout/init.php';
		}
	}

	/**
	 * Redirect users to Stripe for processing payment
	 *
	 * @param $sessionId
	 */
	protected function redirectToStripe($sessionId)
	{
		Factory::getApplication()->getDocument()->addScript('https://js.stripe.com/v3/');

		//Get redirect heading
		$language    = Factory::getLanguage();
		$languageKey = 'EB_WAIT_' . strtoupper(substr($this->name, 3));

		if ($language->hasKey($languageKey))
		{
			$redirectHeading = Text::_($languageKey);
		}
		else
		{
			$redirectHeading = Text::sprintf('EB_REDIRECT_HEADING', $this->getTitle());
		}
		?>
		<div class="payment-heading"><?php
			echo $redirectHeading; ?></div>
		<script type="text/javascript">
			var stripe = Stripe('<?php echo $this->params->get('stripe_public_key'); ?>');

			stripe.redirectToCheckout({
				// Make the id field from the Checkout Session creation API response
				// available to this file, so you can provide it as parameter here
				// instead of the {{CHECKOUT_SESSION_ID}} placeholder.
				sessionId: '<?php echo $sessionId; ?>'
			}).then(function (result) {
				// If `redirectToCheckout` fails due to a browser or network
				// error, display the localized error message to your customer
				// using `result.error.message`.
				alert(result.error.message);
			});
		</script>
		<?php
	}

	/**
	 * Get Stripe Client object
	 *
	 * @return StripeClient
	 */
	private function getStripeClient()
	{
		return new StripeClient($this->params->get('stripe_api_key'));
	}
}